import '../storage_database.dart';
import '../storage_document.dart';
import 'request.dart';
import 'response.dart';

class StorageAPI {
  final StorageDatabase storageDatabase;
  final Future<String>? Function() tokenSource;
  final String apiUrl;
  final bool cashOnOffline;
  final Function(APIResponse respnse)? onReRequestResponse;

  late APIRequest apiRequest;

  StorageAPI({
    required this.storageDatabase,
    required this.tokenSource,
    required this.apiUrl,
    this.cashOnOffline = true,
    this.onReRequestResponse,
  }) {
    apiRequest = APIRequest(storageDatabase, apiUrl, tokenSource);
    storageDatabase.collection('api').set({});
  }

  Future<APIResponse<T>?> request<T>(
    String target,
    RequestType type, {
    bool auth = false,
    Map<String, dynamic>? data,
    bool log = false,
    Function(String reqId)? onNoConnection,
  }) =>
      apiRequest.send(
        target,
        type,
        log: log,
        data: data,
        auth: auth,
        onNoConnection: () async {
          if (cashOnOffline) {
            var reqId = DateTime.now().millisecondsSinceEpoch;
            await storageDatabase.document('api/requests').set({
              '$reqId': {
                'id': reqId,
                'target': target,
                'type': type.type,
                'auth': auth,
                'data': data,
              }
            });
            if (onNoConnection != null) onNoConnection('$reqId');
          }
        },
      );

  Future<APIResponse<T>?> resendRequest<T>(String requestId) async {
    StorageDocument doc =
        storageDatabase.document('api/requests').document(requestId);
    Map requestData = await doc.get();
    APIResponse<T> res = await apiRequest.send(
      requestData['target'],
      RequestTypes.fromString(requestData['type'])!,
      auth: requestData['auth'],
      data: requestData['data'],
    );
    if (res.statusCode != 400) await doc.delete();
    return res;
  }

  Future resendRequests({
    List ids = const [],
    Function(APIResponse respnse)? onResponse,
  }) async {
    if (ids.isEmpty) {
      ids =
          Map.from((await storageDatabase.document('api/requests').get() ?? {}))
              .keys
              .toList();
    }
    for (String id in ids) {
      await resendRequest(id);
    }
  }

  Future clear() async {
    storageDatabase.onClear.add(() async {
      await storageDatabase.collection('api').set({});
    });
    await storageDatabase.collection('api').delete();
  }
}
