import 'dart:convert';
import 'dart:io';

import 'package:http/http.dart' as http;

import '../storage_database.dart';
import 'response.dart';

class APIRequest {
  final StorageDatabase storageDatabase;
  final String apiUrl;
  final Future<String>? Function() tokenSource;

  APIRequest(this.storageDatabase, this.apiUrl, this.tokenSource);

  Map<String, String> get _headers => {
        'Content-Type': 'application/json; charset=UTF-8',
        'Accept': 'application/json',
      };

  Future<String?> get token async => tokenSource();

  Future<String> getUrl(String target, {bool auth = true}) async {
    return '$apiUrl/$target${(auth && (await token) != null ? '?token=${await token}' : '')}';
  }

  Future<APIResponse<T>> send<T>(
    String target,
    RequestType type, {
    bool auth = false,
    Map<String, dynamic>? data,
    bool log = false,
    Function? onNoConnection,
  }) async {
    String url = await getUrl(target, auth: auth);
    if (log) print("[StorageDatabaseAPI] resUrl: $url");
    if (auth && await token == null) {
      return APIResponse<T>(false, "Invalid token", 400);
    }
    int statusCode = 400;
    try {
      http.Response response;
      Uri uri = Uri.parse(url);
      if (type == RequestTypes.post) {
        response = await http.post(
          uri,
          headers: _headers,
          body: jsonEncode(data),
        );
      } else if (type == RequestTypes.put) {
        response = await http.put(
          uri,
          headers: _headers,
          body: jsonEncode(data),
        );
      } else if (type == RequestTypes.patch) {
        response = await http.patch(
          uri,
          headers: _headers,
          body: jsonEncode(data),
        );
      } else if (type == RequestTypes.delete) {
        response = await http.delete(
          uri,
          headers: _headers,
          body: jsonEncode(data),
        );
      } else {
        response = await http.get(uri, headers: _headers);
      }
      statusCode = response.statusCode;
      return APIResponse.fromHttpResponse(response, log: log);
    } on SocketException {
      if (onNoConnection != null) onNoConnection();
      return APIResponse<T>(false, "No Internt Connection", statusCode);
    } catch (e) {
      return APIResponse<T>(false, 'ExceptionError: $e', statusCode);
    }
  }
}

class RequestType {
  final String _type;
  RequestType(this._type);

  String get type => _type;
}

class RequestTypes extends RequestType {
  RequestTypes._(String mode) : super(mode);

  static RequestType get = RequestTypes._('get');
  static RequestType post = RequestTypes._('post');
  static RequestType put = RequestTypes._('put');
  static RequestType patch = RequestTypes._('patch');
  static RequestType delete = RequestTypes._('delete');

  static RequestType? fromString(String type) => type == 'get'
      ? get
      : type == 'post'
          ? post
          : type == 'put'
              ? put
              : type == 'patch'
                  ? patch
                  : type == 'delete'
                      ? delete
                      : null;
}
