import 'dart:convert';
import 'dart:io';
import 'dart:typed_data';

import 'package:flutter/material.dart';
import 'package:http/http.dart' as http;
import 'package:shimmer/shimmer.dart';

import 'explorer_directory.dart';
import 'explorer_file.dart';
import 'storage_explorer.dart';

class ExplorerNetworkFiles {
  final StorageExplorer storageExplorer;
  final ExplorerDirectory networkDirFiles;

  ExplorerNetworkFiles(this.storageExplorer, this.networkDirFiles);

  String encodeUrl(String url) => utf8.fuse(base64).encode(url);
  String decodeUrl(String code) => utf8.fuse(base64).decode(code);

  Future<ExplorerFile> file(String url, {bool refresh = false}) async {
    ExplorerFile file = await networkDirFiles.file(encodeUrl(url));
    if (!file.exists || refresh) {
      await file.setBytes(await downloadFile(Uri.parse(url)));
    }
    return file;
  }

  Future<Uint8List> downloadFile(Uri uri) async =>
      (await http.get(uri)).bodyBytes;

  Widget networkImage(
    String url, {
    double? width,
    double? height,
    double? borderRadius,
    BoxFit fit = BoxFit.cover,
  }) =>
      ExplorerNetworkImage(
        explorerNetworkFiles: this,
        url: url,
        width: width,
        height: height,
        borderRadius: borderRadius,
        fit: fit,
      );
}

class ExplorerNetworkImage extends StatefulWidget {
  final String url;
  final ExplorerNetworkFiles explorerNetworkFiles;
  final double? width, height, borderRadius;
  final BoxFit fit;

  const ExplorerNetworkImage({
    Key? key,
    required this.explorerNetworkFiles,
    required this.url,
    this.width,
    this.height,
    this.borderRadius,
    this.fit = BoxFit.cover,
  }) : super(key: key);

  @override
  State<ExplorerNetworkImage> createState() => _ExplorerNetworkImageState();
}

class _ExplorerNetworkImageState extends State<ExplorerNetworkImage> {
  bool isLoading = true;
  bool hasError = false;
  File? image;

  @override
  void initState() {
    init();
    super.initState();
  }

  init() async {
    try {
      image = (await widget.explorerNetworkFiles.file(widget.url)).ioFile;
      setState(() {
        isLoading = false;
      });
    } catch (e) {
      hasError = true;
    }
  }

  @override
  Widget build(BuildContext context) {
    return SizedBox(
      width: widget.width,
      height: widget.height,
      child: isLoading
          ? Shimmer.fromColors(
              baseColor: Color.fromARGB(255, 168, 168, 168),
              highlightColor: Color.fromARGB(255, 236, 236, 236),
              child: Container(
                decoration: BoxDecoration(
                  borderRadius: BorderRadius.circular(
                    widget.borderRadius ?? 0,
                  ),
                  color: Color.fromARGB(255, 236, 236, 236),
                ),
              ),
            )
          : Container(
              decoration: BoxDecoration(
                color: Color.fromARGB(255, 168, 168, 168),
                borderRadius: BorderRadius.circular(
                  widget.borderRadius ?? 0,
                ),
                image: image != null
                    ? DecorationImage(image: FileImage(image!), fit: widget.fit)
                    : null,
              ),
              child: hasError
                  ? Icon(
                      Icons.broken_image,
                      color: Color.fromARGB(255, 236, 236, 236),
                      size: (widget.width != null ? (widget.width! / 2) : null),
                    )
                  : null,
            ),
    );
  }
}
